@@ -34,6 +34,7 @@
 #include "gpm-marshal.h"
 #include "gpm-engine.h"
 #include "gpm-stock-icons.h"
+#include "gpm-prefs-server.h"
 #include "gpm-phone.h"
 
 static void     gpm_engine_finalize   (GObject	  *object);
@@ -50,7 +51,7 @@ struct GpmEnginePrivate
 	GPtrArray		*array;
 	GpmPhone		*phone;
 	GpmIconPolicy		 icon_policy;
-	GIcon			*previous_icon;
+	gchar			*previous_icon;
 	gchar			*previous_summary;
 
 	gboolean		 use_time_primary;
@@ -74,7 +75,6 @@ enum {
 	DISCHARGING,
 	LOW_CAPACITY,
 	PERHAPS_RECALL,
-	DEVICES_CHANGED,
 	LAST_SIGNAL
 };
 
@@ -281,7 +281,7 @@ gpm_engine_get_summary (GpmEngine *engine)
  *
  * Returns the icon
  **/
-static GIcon *
+static gchar *
 gpm_engine_get_icon_priv (GpmEngine *engine, UpDeviceKind device_kind, GpmEngineWarning warning, gboolean use_state)
 {
 	guint i;
@@ -312,15 +312,15 @@ gpm_engine_get_icon_priv (GpmEngine *engine, UpDeviceKind device_kind, GpmEngine
 		if (kind == device_kind && is_present) {
 			if (warning != GPM_ENGINE_WARNING_NONE) {
 				if (warning_temp == warning)
-					return gpm_upower_get_device_icon (device, TRUE);
+					return gpm_upower_get_device_icon (device);
 				continue;
 			}
 			if (use_state) {
 				if (state == UP_DEVICE_STATE_CHARGING || state == UP_DEVICE_STATE_DISCHARGING)
-					return gpm_upower_get_device_icon (device, TRUE);
+					return gpm_upower_get_device_icon (device);
 				continue;
 			}
-			return gpm_upower_get_device_icon (device, TRUE);
+			return gpm_upower_get_device_icon (device);
 		}
 	}
 	return NULL;
@@ -331,10 +331,10 @@ gpm_engine_get_icon_priv (GpmEngine *engine, UpDeviceKind device_kind, GpmEngine
  *
  * Returns the icon
  **/
-GIcon *
+gchar *
 gpm_engine_get_icon (GpmEngine *engine)
 {
-	GIcon *icon = NULL;
+	gchar *icon = NULL;
 
 	g_return_val_if_fail (GPM_IS_ENGINE (engine), NULL);
 
@@ -412,8 +412,9 @@ gpm_engine_get_icon (GpmEngine *engine)
 		return NULL;
 	}
 
-	/* do not show an icon */
-	return NULL;
+	/* we fallback to the ac_adapter icon */
+	egg_debug ("Using fallback");
+	return g_strdup (GPM_STOCK_AC_ADAPTER);
 }
 
 /**
@@ -422,7 +423,7 @@ gpm_engine_get_icon (GpmEngine *engine)
 static gboolean
 gpm_engine_recalculate_state_icon (GpmEngine *engine)
 {
-	GIcon *icon;
+	gchar *icon;
 
 	g_return_val_if_fail (engine != NULL, FALSE);
 	g_return_val_if_fail (GPM_IS_ENGINE (engine), FALSE);
@@ -437,29 +438,31 @@ gpm_engine_recalculate_state_icon (GpmEngine *engine)
 		egg_debug ("** EMIT: icon-changed: none");
 		g_signal_emit (engine, signals [ICON_CHANGED], 0, NULL);
 
-		g_object_unref (engine->priv->previous_icon);
+		g_free (engine->priv->previous_icon);
 		engine->priv->previous_icon = NULL;
 		return TRUE;
 	}
 
 	/* no icon before, now icon */
 	if (engine->priv->previous_icon == NULL) {
+		egg_debug ("** EMIT: icon-changed: %s", icon);
 		g_signal_emit (engine, signals [ICON_CHANGED], 0, icon);
-		engine->priv->previous_icon = g_object_ref (icon);
+		engine->priv->previous_icon = icon;
 		return TRUE;
 	}
 
 	/* icon before, now different */
-	if (!g_icon_equal (engine->priv->previous_icon, icon)) {
-		g_object_unref (engine->priv->previous_icon);
-		engine->priv->previous_icon = g_object_ref (icon);
+	if (strcmp (engine->priv->previous_icon, icon) != 0) {
+		g_free (engine->priv->previous_icon);
+		engine->priv->previous_icon = icon;
+		egg_debug ("** EMIT: icon-changed: %s", icon);
 		g_signal_emit (engine, signals [ICON_CHANGED], 0, icon);
 		return TRUE;
 	}
 
 	egg_debug ("no change");
 	/* nothing to do */
-	g_object_unref (icon);
+	g_free (icon);
 	return FALSE;
 }
 
@@ -504,8 +507,6 @@ gpm_engine_recalculate_state (GpmEngine *engine)
 
 	gpm_engine_recalculate_state_icon (engine);
 	gpm_engine_recalculate_state_summary (engine);
-
-	g_signal_emit (engine, signals [DEVICES_CHANGED], 0);
 }
 
 /**
@@ -586,14 +587,8 @@ gpm_engine_get_composite_device (GpmEngine *engine, UpDevice *original_device)
 	GPtrArray *array;
 	UpDevice *device;
 	UpDeviceKind kind;
-	UpDeviceKind original_kind;
 	guint i;
 
-	/* get the type of the original device */
-	g_object_get (original_device,
-		      "kind", &original_kind,
-		      NULL);
-
 	/* find out how many batteries in the system */
 	array = engine->priv->array;
 	for (i=0;i<array->len;i++) {
@@ -601,7 +596,7 @@ gpm_engine_get_composite_device (GpmEngine *engine, UpDevice *original_device)
 		g_object_get (device,
 			      "kind", &kind,
 			      NULL);
-		if (kind == original_kind)
+		if (kind == UP_DEVICE_KIND_BATTERY)
 			battery_devices++;
 	}
 
@@ -626,10 +621,10 @@ static UpDevice *
 gpm_engine_update_composite_device (GpmEngine *engine, UpDevice *original_device)
 {
 	guint i;
-	gdouble percentage;
-	gdouble energy;
-	gdouble energy_full;
-	gdouble energy_rate;
+	gdouble percentage = 0.0;
+	gdouble energy = 0.0;
+	gdouble energy_full = 0.0;
+	gdouble energy_rate = 0.0;
 	gdouble energy_total = 0.0;
 	gdouble energy_full_total = 0.0;
 	gdouble energy_rate_total = 0.0;
@@ -643,18 +638,12 @@ gpm_engine_update_composite_device (GpmEngine *engine, UpDevice *original_device
 	UpDevice *device;
 	UpDeviceState state;
 	UpDeviceKind kind;
-	UpDeviceKind original_kind;
 	gboolean debug;
 	gchar *text;
 
 	/* are we printing to console? */
 	debug = egg_debug_enabled ();
 
-	/* get the type of the original device */
-	g_object_get (original_device,
-		      "kind", &original_kind,
-		      NULL);
-
 	/* update the composite device */
 	array = engine->priv->array;
 	for (i=0;i<array->len;i++) {
@@ -666,7 +655,7 @@ gpm_engine_update_composite_device (GpmEngine *engine, UpDevice *original_device
 			      "energy-full", &energy_full,
 			      "energy-rate", &energy_rate,
 			      NULL);
-		if (kind != original_kind)
+		if (kind != UP_DEVICE_KIND_BATTERY)
 			continue;
 
 		if (debug) {
@@ -827,8 +816,12 @@ static gboolean
 gpm_engine_coldplug_idle_cb (GpmEngine *engine)
 {
 	guint i;
-	GPtrArray *array = NULL;
+	GPtrArray *array;
+	gboolean has_battery = FALSE;
+	gboolean has_ups = FALSE;
+	GpmPrefsServer *prefs_server;
 	UpDevice *device;
+	UpDeviceKind kind;
 	gboolean ret;
 	GError *error = NULL;
 
@@ -842,6 +835,31 @@ gpm_engine_coldplug_idle_cb (GpmEngine *engine)
 		g_error_free (error);
 		goto out;
 	}
+	engine->priv->array = up_client_get_devices (engine->priv->client);
+
+	/* do we have specific device types? */
+	array = engine->priv->array;
+	for (i=0;i<array->len;i++) {
+		device = g_ptr_array_index (engine->priv->array, i);
+
+		/* get device properties */
+		g_object_get (device,
+			      "kind", &kind,
+			      NULL);
+
+		if (kind == UP_DEVICE_KIND_BATTERY)
+			has_battery = TRUE;
+		else if (kind == UP_DEVICE_KIND_UPS)
+			has_ups = TRUE;
+	}
+
+	/* only show the battery prefs section if we have batteries */
+	prefs_server = gpm_prefs_server_new ();
+	if (has_battery)
+		gpm_prefs_server_set_capability (prefs_server, GPM_PREFS_SERVER_BATTERY);
+	if (has_ups)
+		gpm_prefs_server_set_capability (prefs_server, GPM_PREFS_SERVER_UPS);
+	g_object_unref (prefs_server);
 
 	/* connected mobile phones */
 	gpm_phone_coldplug (engine->priv->phone);
@@ -849,15 +867,12 @@ gpm_engine_coldplug_idle_cb (GpmEngine *engine)
 	gpm_engine_recalculate_state (engine);
 
 	/* add to database */
-	array = up_client_get_devices (engine->priv->client);
 	for (i=0;i<array->len;i++) {
-		device = g_ptr_array_index (array, i);
+		device = g_ptr_array_index (engine->priv->array, i);
 		gpm_engine_device_add (engine, device);
 		gpm_engine_check_recall (engine, device);
 	}
 out:
-	if (array != NULL)
-		g_ptr_array_unref (array);
 	/* never repeat */
 	return FALSE;
 }
@@ -966,50 +981,6 @@ gpm_engine_get_devices (GpmEngine *engine)
 	return g_ptr_array_ref (engine->priv->array);
 }
 
-/**
- * gpm_engine_get_primary_device:
- *
- * Return value: the #UpDevice, free with g_object_unref()
- **/
-UpDevice *
-gpm_engine_get_primary_device (GpmEngine *engine)
-{
-	guint i;
-	UpDevice *device = NULL;
-	UpDevice *device_tmp;
-	UpDeviceKind kind;
-	UpDeviceState state;
-	gboolean is_present;
-
-	for (i=0; i<engine->priv->array->len; i++) {
-		device_tmp = g_ptr_array_index (engine->priv->array, i);
-
-		/* get device properties */
-		g_object_get (device_tmp,
-			      "kind", &kind,
-			      "state", &state,
-			      "is-present", &is_present,
-			      NULL);
-
-		/* not present */
-		if (!is_present)
-			continue;
-
-		/* not discharging */
-		if (state != UP_DEVICE_STATE_DISCHARGING)
-			continue;
-
-		/* not battery */
-		if (kind != UP_DEVICE_KIND_BATTERY)
-			continue;
-
-		/* use composite device to cope with multiple batteries */
-		device = g_object_ref (gpm_engine_get_composite_device (engine, device_tmp));
-		break;
-	}
-	return device;
-}
-
 /**
  * phone_device_added_cb:
  **/
@@ -1190,8 +1161,8 @@ gpm_engine_class_init (GpmEngineClass *klass)
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (GpmEngineClass, icon_changed),
-			      NULL, NULL, g_cclosure_marshal_VOID__OBJECT,
-			      G_TYPE_NONE, 1, G_TYPE_ICON);
+			      NULL, NULL, g_cclosure_marshal_VOID__STRING,
+			      G_TYPE_NONE, 1, G_TYPE_STRING);
 	signals [SUMMARY_CHANGED] =
 		g_signal_new ("summary-changed",
 			      G_TYPE_FROM_CLASS (object_class),
@@ -1249,13 +1220,6 @@ gpm_engine_class_init (GpmEngineClass *klass)
 			      G_STRUCT_OFFSET (GpmEngineClass, charge_critical),
 			      NULL, NULL, g_cclosure_marshal_VOID__POINTER,
 			      G_TYPE_NONE, 1, G_TYPE_POINTER);
-	signals [DEVICES_CHANGED] =
-		g_signal_new ("devices-changed",
-			      G_TYPE_FROM_CLASS (object_class),
-			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (GpmEngineClass, devices_changed),
-			      NULL, NULL, g_cclosure_marshal_VOID__VOID,
-			      G_TYPE_NONE, 0);
 }
 
 /**
@@ -1278,8 +1242,7 @@ gpm_engine_finalize (GObject *object)
 	g_object_unref (engine->priv->phone);
 	g_object_unref (engine->priv->battery_composite);
 
-	if (engine->priv->previous_icon != NULL)
-		g_object_unref (engine->priv->previous_icon);
+	g_free (engine->priv->previous_icon);
 	g_free (engine->priv->previous_summary);
 
 	G_OBJECT_CLASS (gpm_engine_parent_class)->finalize (object);
